from unittest import result
import mugrade
import numpy as np

def objects_equal(value,ref):
    """ Test if two objects are equal according to our autograding rules.
    We have to do this manually so that numpy arrays are compard properly. """

    try:
        if isinstance(ref, dict):
            if sorted(value.keys()) != sorted(ref.keys()):
                return False
            for k,v in ref.items():
                if not objects_equal(v, value[k]):
                    return False
            return True
    
        elif isinstance(ref, list) or isinstance(ref, tuple):
            if len(value) != len(ref):
                return False
            for a,b in zip(ref, value):
                if not objects_equal(a, b):
                    return False
            return True
    
        elif isinstance(ref, np.ndarray):
            if value.shape != ref.shape:
                return False
            if not np.allclose(ref, value, atol=1e-8):
                return False
            return True
          
        elif isinstance(ref, float) or isinstance(ref, np.float32):
          return np.allclose(value, ref)
    
        else:
            return value == ref
    except:
        return False

def test_load_census_data():
    df = load_census_data()
    with mugrade.test:
        assert set(df.columns) == {'work_class', 'race', 'final_weight', 'sex', 'capital_loss', 'income', 'education', 'native_country', 'marital_status', 'relationship', 'education_num', 'hours_per_week', 'capital_gain', 'age', 'occupation'}
    with mugrade.test:
        assert dict(df.dtypes) == {'age': np.dtype(int), 'work_class': np.dtype(object), 'final_weight': np.dtype(int), 'education': np.dtype(object), 'education_num': np.dtype(int), 'marital_status': np.dtype(object), 'occupation': np.dtype(object), 'relationship': np.dtype(object), 'race': np.dtype(object), 'sex': np.dtype(object), 'capital_gain': np.dtype(int), 'capital_loss': np.dtype(int), 'hours_per_week': np.dtype(int), 'native_country': np.dtype(object), 'income': np.dtype(object)}
    with mugrade.test:
        assert list(df["age"].values[:10]) == [39, 50, 38, 53, 28, 37, 49, 52, 31, 42]

def submit_load_census_data():
    df = load_census_data()
    mugrade.submit(df["hours_per_week"].mean())
    mugrade.submit(list(df["education"].values[:10]))
    mugrade.submit(len(df))

def test_CategoricalDistribution():
    with mugrade.test:
        assert objects_equal
    X_toy = pd.Series(["A"]*100 + ["B"]*50 + ["C"]*10)
    Y_toy = pd.Series([1]*100 + [2]*50 + [3]*10)

    dist_toy1 = CategoricalDistribution(X_toy, Y_toy)
    with mugrade.test:
        assert objects_equal(dist_toy1.log_likelihood("A", 1), -0.019608471388376313)
    with mugrade.test:
        assert objects_equal(dist_toy1.log_likelihood("A", 2), -3.970291913552122)

    dist_toy2 = CategoricalDistribution(X_toy, Y_toy, alpha=0.5)
    with mugrade.test:
        assert objects_equal(dist_toy2.log_likelihood("C", 3), -0.09097177820572676)

    df = load_census_data()
    dist_edu_income = CategoricalDistribution(df["education"], df["income"])
    with mugrade.test:
        assert objects_equal(dist_edu_income.log_likelihood("HS-grad", ">50K"), -1.536907092802484)
    with mugrade.test:
        assert objects_equal(dist_edu_income.log_likelihood("Doctorate", "<=50K"), -5.464449552033739)

def submit_CategoricalDistribution():
    X_toy = pd.Series(["A"]*100 + ["B"]*50 + ["C"]*10)
    Y_toy = pd.Series([1]*100 + [2]*50 + [3]*10)

    dist_toy1 = CategoricalDistribution(X_toy, Y_toy)
    mugrade.submit(dist_toy1.log_likelihood("B", 2))

    dist_toy2 = CategoricalDistribution(X_toy, Y_toy, alpha=0.5)
    mugrade.submit(dist_toy2.log_likelihood("B", 1))

    df = load_census_data()
    mugrade.submit(CategoricalDistribution(df["marital_status"], df["income"], alpha=0.5).log_likelihood("Never-married", ">50K"))
    mugrade.submit(CategoricalDistribution(df["occupation"], df["education"]).log_likelihood("Prof-specialty", "Doctorate"))

def test_GaussianDistribution():
    X_toy = pd.Series([1]*10 + [2]*20 + [3]*20)
    Y_toy = pd.Series(["A"]*15 + ["B"]*35)
    dist_toy = GaussianDistribution(X_toy, Y_toy)
    with mugrade.test:
        assert objects_equal(dist_toy.log_likelihood(2, "A"), -1.1668998348165358)
    with mugrade.test:
        assert objects_equal(dist_toy.log_likelihood(1, "B"), -5.257148375710028)

    df = load_census_data()
    dist_age_income = GaussianDistribution(df["age"], df["income"])
    with mugrade.test:
        assert objects_equal(dist_age_income.log_likelihood(30,">50K"), -4.171981348529774)
    with mugrade.test:
        assert objects_equal(dist_age_income.log_likelihood(35,"<=50K"), -3.526114697385937)

def submit_GaussianDistribution():
    X_toy = pd.Series([1]*10 + [2]*20 + [3]*20)
    Y_toy = pd.Series(["A"]*15 + ["B"]*35)
    dist_toy = GaussianDistribution(X_toy, Y_toy)
    mugrade.submit(dist_toy.log_likelihood(3, "B"))
    mugrade.submit(dist_toy.log_likelihood(1, "A"))

    df = load_census_data()
    dist_hours_income = GaussianDistribution(df["hours_per_week"], df["income"])
    mugrade.submit(dist_hours_income.log_likelihood(42,">50K"))
    mugrade.submit(dist_hours_income.log_likelihood(42,"<=50K"))

def test_NaiveBayesClassifier():
    df = load_census_data()

    clf_income1 = NaiveBayesClassifier(df.drop("income", axis=1), df["income"])
    with mugrade.test:
        assert objects_equal(clf_income1.probabilities(df.iloc[0]), {'<=50K': 0.9722830757698078, '>50K': 0.027716924230192314})

    clf_income2 = NaiveBayesClassifier(df.drop("income", axis=1), df["income"], alpha=100)
    with mugrade.test:
        assert objects_equal(clf_income2.probabilities(df.iloc[1]), {'<=50K': 0.8200350959564986, '>50K': 0.17996490404350143})

    clf_relationship = NaiveBayesClassifier(df.drop("relationship", axis=1), df["relationship"])
    with mugrade.test:
        assert objects_equal(clf_relationship.probabilities(df.iloc[3]), {'Not-in-family': 0.0016939969120619229, 'Husband': 0.425560427449148, 'Wife': 2.5041833377701984e-05, 'Own-child': 0.004261209109891672, 'Unmarried': 0.0004774844846792824, 'Other-relative': 0.5679818402108414})

def submit_NaiveBayesClassifier():
    df = load_census_data()

    clf_income1 = NaiveBayesClassifier(df.drop("income", axis=1), df["income"])
    mugrade.submit(clf_income1.probabilities(df.iloc[3]))

    clf_income2 = NaiveBayesClassifier(df.drop("income", axis=1), df["income"], alpha=100)
    mugrade.submit(clf_income2.probabilities(df.iloc[3]))

    clf_sex = NaiveBayesClassifier(df.drop("sex", axis=1), df["sex"])
    mugrade.submit(clf_sex.probabilities(df.iloc[3]))
